# Copyright 2022 Amethyst Reese
# Licensed under the MIT license

# Generated by stdlibs/fetch.py

from typing import List

KNOWN_VERSIONS: List[str] = [
    "2.3",
    "2.4",
    "2.5",
    "2.6",
    "2.7",
    "3.0",
    "3.1",
    "3.2",
    "3.3",
    "3.4",
    "3.5",
    "3.6",
    "3.7",
    "3.8",
    "3.9",
    "3.10",
    "3.11",
]
"""All supported Python major releases"""
